// $Id: test-insertid.C,v 1.4 2004/02/15 13:47:19 dvermeir Exp $

#include <string>
#include <iostream>
#include <fstream>

#include <dvutil/props.h>
#include <dvmysql/mysqldb.h>

using namespace std;
using namespace Dv;

bool	verbose(false);

// a helper function that executes a text as a databse command and
// aborts the program if the execution failed
void
do_simple_command(Sql::Db& db,const char* txt,bool care=true) {
Sql::Command u(db,txt);

u.exec();

if (!u && care)
  throw runtime_error(u.err());
}

int
main(int argc,char* argv[]) {
try {
  static const string CONFIG("tmysql.config");
  ifstream	ifs(CONFIG.c_str());
  if (!ifs)
    throw runtime_error(CONFIG+": cannot open");
  
  Dv::Util::Props	config(ifs);
  string dictfn(config["dict"].str());

  // open a connection to the database
  
  MySql::Db	db(config,"tmysql::");
  
  // create a simple table and an index; the table stores <number,word> pairs
  
  do_simple_command(db, "drop table test",false);
  do_simple_command(db, "create table test "
                        "  ( number integer auto_increment primary key, word char(32) binary not null)");
  do_simple_command(db, "create index indx on test (word)");
  
  // the test consists of inserting words from /usr/dict/words into the database
  // where each word is associated with its position in the file; then all words are
  // retrieved and the result is verified
  {
  
  ifstream	is(dictfn.c_str()); // open the file
  
  if (!is)
    throw runtime_error(dictfn+": cannot open");
  
  string		word;
  
  for (unsigned int i=1;(is>>word);++i) { 
    // while we can read a word from the file
    Sql::Command u(db,"insert into test(word) values (");
    // insert it into the database
    u << word << ")";
    if (!u.exec())
      throw runtime_error(u.err());
    unsigned int id(u.insertid());
    if (i!=id) {
      cerr << "insertid = " << id << ", expected " << i << endl;
      throw runtime_error(u.err());
      }
    if (i%1000==0)
      cerr << ".";
    }
  cerr << endl;
  }
  
  return 0; // everything checks out, return 0 to confirm successful test
  }
catch (exception& e) {
  cerr << e.what() << endl;
  return 1;
  }
}
