// $Id: test-mysql.C,v 1.3 2004/02/15 13:47:19 dvermeir Exp $

#include <string>
#include <iostream>
#include <fstream>

#include <dvutil/props.h>
#include <dvmysql/mysqldb.h>

using namespace std;
using namespace Dv;

bool	verbose(false);

// a helper function that executes a text as a databse command and
// aborts the program if the execution failed
void
do_simple_command(Sql::Db& db,const char* txt,bool care=true) {
Sql::Command u(db,txt);

u.exec();

if (!u && care)
  throw runtime_error(u.err());
}

int
main(int argc,char* argv[]) {
try {
  static const string CONFIG("tmysql.config");
  ifstream	ifs(CONFIG.c_str());
  if (!ifs)
    throw runtime_error(CONFIG+": cannot open");
  
  Dv::Util::Props	config(ifs);
  string dictfn(config["dict"].str());

  // open a connection to the database
  
  MySql::Db	db(config,"tmysql::");
  
  // create a simple table and an index; the table stores <number,word> pairs
  
  do_simple_command(db, "drop table test",false);
  do_simple_command(db, "create table test "
                        "  ( number integer, word char(32) binary not null)");
  do_simple_command(db, "create index indx on test (word)");
  
  // the test consists of inserting words from /usr/dict/words into the database
  // where each word is associated with its position in the file; then all words are
  // retrieved and the result is verified
  {
  
  ifstream	is(dictfn.c_str()); // open the file
  
  if (!is)
    throw runtime_error(dictfn+": cannot open");
  
  string		word;
  
  for (unsigned int i=0;(is>>word);++i) { 
    // while we can read a word from the file
    Sql::Command u(db,"insert into test values (");
    // insert it into the database
    u << static_cast<int>(i) << "," << word << ")";
    if (!u.exec())
      throw runtime_error(u.err());
    if (i%1000==0)
      cerr << ".";
    }
  cerr << endl;
  }
  
  // check: read each word from the file, then retrieve the word
  // from the database and verify that it has the appropriate number
  {
  ifstream		is(dictfn.c_str());
  
  if (!is)
    throw runtime_error(dictfn+": cannot reopen");
  
  string	word;
  
  for (unsigned int i=0;(is>>word);++i) { 
    // while we can read the i'th word from the file
    Sql::Command	q(db,"select number from test where word = ");
    q << word;
    if (!q.exec())
      throw runtime_error(q.err());
  
    if (q.nrows()!=1) { // only 1 row should have been retrieved!
      cout << q.text() << " retrieved " 
      	 << q.nrows() << " rows instead of 1";
      return 1; // error exit
      }
    // get result, there is only one column
    vector<string>	result = *(q.begin());
  
    if (atoi(result[0].c_str())!=static_cast<int>(i)) {
       // wrong number has been retrieved
      if (verbose)
        cout << q.text() << " retrieved " 
             << q.begin()(0) << " instead of " << i;
      return 1;
      }
    if (i%1000==0)
      cerr << ".";
    }
  cerr << endl;
  }
  return 0; // everything checks out, return 0 to confirm successful test
  }
catch (exception& e) {
  cerr << e.what() << endl;
  return 1;
  }
}
